package com.ibm.tools.attach.javaSE;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2009, 2011  All Rights Reserved
 */

import java.lang.reflect.Field;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Timer;

/**
 * Allows us to ensure the timer thread is dead,
 *
 */
public class FilelockTimer extends Timer {

	volatile Field timerThreadField;

	/**
	 * @param name Thread name
	 */
	public FilelockTimer(String name) {
		super(name, true);

		timerThreadField = AccessController.doPrivileged(new PrivilegedAction<Field>() {
			public Field run() {
				try {
					final Field impl = Timer.class.getDeclaredField("impl"); //$NON-NLS-1$
					impl.setAccessible(true);
					return impl;
				} catch (Exception e) {
					IPC.logMessage("FilelockTimer: get field exception "+e.getClass().getName()+":"+e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
				}
				return null;
			}
		});
	}

	public void cancel() {
		super.cancel();
		if (null != timerThreadField) {
			try {
				Thread timerThread = (Thread) timerThreadField.get(this);
				if (null != timerThread) {
					timerThread.join();
				}
			} catch (Exception e) {
				IPC.logMessage("Exception in FilelockTimer.cancel: ", e.getClass().getName()+":"+e.getMessage()); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}
}

